-- Filename:     fa_4bit.vhd
-- Filetype:     VHDL Design File
-- Date:         9 mar 2014
-- Update:       -
-- Description:  Description of an 4-bit adder
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.2
-- Copyright:    (c)2014, De Haagse Hogeschool
--
-- This is a description of a 4-bit full adder circuit.
-- This example uses the numeric_std package for addition
-- since it is recommended. Please note that the type
-- unsigned is defined the same as a std_logic_vector, but
-- because VHDL does strict typing, it is not allowed to
-- mix the two types in an expression.

-- Use the std_logic_1164 and numeric_std libraries
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- The entity
entity fa_4bit is
    port (a, b : in unsigned(3 downto 0);
          cin  : in std_logic;
          s    : out unsigned(3 downto 0);
          cout : out std_logic;
          zero : out std_logic;
          over : out std_logic
    );
end entity fa_4bit;

-- The architecture
architecture arith of fa_4bit is
-- Internal 5-bit result
signal s_int : unsigned(4 downto 0);
begin
    -- Addition
    s_int <= ('0' & a) + ('0' & b) + ("0" & cin);
    --s_int <= ('0' & a) + ('0' & b) + ("" & cin);
 
    -- Map lower 4 bits of result to sum
    s     <= s_int(3 downto 0);
 
    -- Carry out, high bit
    cout  <= s_int(4);
 
    -- Zero flag
    zero  <= '1' when s_int(3 downto 0) = "0000" else '0';
 
    -- Overflow flag
    over <= (not a(3) and not b(3) and s_int(3)) or (a(3) and b(3) and not s_int(3));\
	 --over <= '1' when (a(3) = b(3)) and (a(3) /= s_int(3)) else '0';

end arith;

