# Filename:     tb_fa_4bit.do
# Filetype:     Modelsim Command File
# Date:         9 mar 2014
# Update:       -
# Description:  Command file for simulating the 4-bit full adder
# Author:       J. op den Brouw
# State:        demo
# Error:        -
# Version:      1.2
# Copyright:    (c)2014, De Haagse Hogeschool

# Set transcript on
transcript on

# Recreate the work directory and map to work
if {[file exists rtl_work]} {
    vdel -lib rtl_work -all
}
vlib rtl_work
vmap work rtl_work

# Find out if we're started through Quartus or by hand
#(or by using an exec in the Tcl window in Quartus).
# Quartus has the annoying property that it will start
# Modelsim from a directory called "simulation/modelsim".
# The design and the testbench are located in the project
# root, so we've to compensate for that.
if [ string match "*simulation/modelsim" [pwd] ] { 
	set prefix "../../"
	puts "Running Modelsim from Quartus..."
} else {
	set prefix ""
	puts "Running Modelsim..."
}

# Compile full adder and its testbench, please note that
# the design and its testbench are located in the project
# root, but the simulator starts in directory
# <project_root>/simulation/modelsim, so we have to compensate
# for that.
vcom -93 -work work ${prefix}fa_4bit.vhd
vcom -93 -work work ${prefix}tb_fa_4bit.vhd

# Start the simulator with 1 ns time resolution
vsim -t 1ns -L rtl_work -L work -voptargs="+acc" tb_fa_4bit

# Log all signals in the design, good if the number
# of signals is small.
add log -r *

# Add all toplevel signals to the list view
# Add a number of signals of the simulated design
add list a
add list b
add list cin
add list dut/s_int
add list s
add list cout
add list zero

# Add all toplevel signals to the wave view
# Add a number of signals inside the simulated design
add wave -divider "Inputs"
add wave a
add wave b
add wave cin
add wave -divider "Internals"
add wave dut/s_int
add wave -divider "Outputs"
add wave s
add wave cout
add wave zero
add wave over

# Open Structure, List and Waveform window
view structure
view list
view wave

# Run simulation until no more events on stimuli
run -all

# Fill up the waveform in the window
wave zoom full
