-- Filename:     tb_fa_4bit.vhd
-- Filetype:     VHDL Test Bench File
-- Date:         6 dec 2012
-- Update:       -
-- Description:  Test bench for fa_4bit
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.1
-- Copyright:    (c)2012, De Haagse Hogeschool
--
-- This is a test bench for the 4-bit full adder
-- VHDL description.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- Empty entity
entity tb_fa_4bit is
end entity tb_fa_4bit;

-- The architecture (sim)
architecture sim of tb_fa_4bit is
-- These are the signals we want to monitor
signal a, b : unsigned(3 downto 0);
signal cin  : std_logic;
signal s    : unsigned(3 downto 0);
signal cout : std_logic;
signal zero : std_logic;
signal over : std_logic;

-- The 4-bot full adder...
component fa_4bit is
	port (a, b : in unsigned(3 downto 0);
			cin  : in std_logic;
			s    : out unsigned(3 downto 0);
			cout : out std_logic;
			zero : out std_logic;
			over : out std_logic
			);
end component fa_4bit;

begin

	-- Instantiate a 4-bit full adder and map the port signals
	-- to the monitoring signals
	dut : fa_4bit
	port map (a => a, b => b, cin => cin, s => s, cout => cout, zero => zero, over => over);

   -- The stimuli. We loop through all combinations
	process is
	begin
		-- Loop a from 0 (0000) to 15 (1111)
		for i in 0 to 15 loop
			a <= to_unsigned(i, 4);
			-- Loop b from 0 (0000) to 15 (1111)
			for j in 0 to 15 loop
				b <= to_unsigned(j, 4);
				-- Loop cin from 0 (0) to 1 (1)
				for k in 0 to 1 loop
					if k = 0 then cin <= '0'; else cin <= '1'; end if;
					-- Wait for a while, so we can see the signals.
					wait for 10 ns;
				end loop;
			end loop;
		end loop;
		
		-- Wait forever
		wait;
	end process;
	
end architecture sim;
